@@ -6,6 +6,7 @@ from multiprocessing import Pool
 
 import comet_maths as cm
 import numpy as np
+
 import punpy.utilities.utilities as util
 
 """___Authorship___"""
@@ -75,6 +76,8 @@ class MCPropagation:
         :type param_fixed: list of bools, optional
         :param corr_between: correlation matrix (n,n) between input quantities, defaults to None
         :type corr_between: array, optional
+        :param samples: allows to provide a Monte Carlo sample previously generated. This sample of input quantities will be used instead of generating one from the uncertainties and error-correlation. Defaults to None
+        :type samples: list[array], optional
         :param return_corr: set to True to return correlation matrix of measurand, defaults to False
         :type return_corr: bool, optional
         :param return_samples: set to True to return generated samples, defaults to False
@@ -154,6 +157,8 @@ class MCPropagation:
         :type param_fixed: list of bools, optional
         :param corr_between: correlation matrix (n,n) between input quantities, defaults to None
         :type corr_between: array, optional
+        :param samples: allows to provide a Monte Carlo sample previously generated. This sample of input quantities will be used instead of generating one from the uncertainties and error-correlation. Defaults to None
+        :type samples: list[array], optional
         :param return_corr: set to True to return correlation matrix of measurand, defaults to False
         :type return_corr: bool, optional
         :param return_samples: set to True to return generated samples, defaults to False
@@ -197,6 +202,87 @@ class MCPropagation:
             refyvar=refyvar,
         )
 
+    def propagate_cov(
+        self,
+        func,
+        x,
+        cov_x,
+        param_fixed=None,
+        corr_between=None,
+        samples=None,
+        return_corr=True,
+        return_samples=False,
+        repeat_dims=-99,
+        corr_axis=-99,
+        fixed_corr_var=False,
+        output_vars=1,
+        PD_corr=True,
+        refyvar=0,
+    ):
+        """
+        Propagate uncertainties with given covariance matrix through measurement function with n input quantities.
+        Input quantities can be floats, vectors (1d-array) or images (2d-array).
+        The covariance matrix can represent the full covariance matrix between all measurements in all dimensions.
+        Alternatively if there are repeated measurements specified in repeat_dims, the covariance matrix is given
+        for the covariance along the dimension that is not one of the repeat_dims.
+
+        :param func: measurement function
+        :type func: function
+        :param x: list of input quantities (usually numpy arrays)
+        :type x: list[array]
+        :param cov_x: list of covariance matrices on input quantities (usually numpy arrays). In case the input quantity is an array of shape (m,o), the covariance matrix  is typically given as an array of shape (m*o,m*o).
+        :type cov_x: list[array]
+        :param param_fixed: when repeat_dims>=0, set to true or false to indicate for each input quantity whether it has repeated measurements that should be split (param_fixed=False) or whether the input is fixed (param fixed=True), defaults to None (no inputs fixed).
+        :type param_fixed: list of bools, optional
+        :param corr_between: covariance matrix (n,n) between input quantities, defaults to None
+        :type corr_between: array, optional
+        :param samples: allows to provide a Monte Carlo sample previously generated. This sample of input quantities will be used instead of generating one from the uncertainties and error-correlation. Defaults to None
+        :type samples: list[array], optional
+        :param return_corr: set to True to return correlation matrix of measurand, defaults to True
+        :type return_corr: bool, optional
+        :param return_samples: set to True to return generated samples, defaults to False
+        :type return_samples: bool, optional
+        :param repeat_dims: set to positive integer(s) to select the axis which has repeated measurements. The calculations will be performed seperately for each of the repeated measurments and then combined, in order to save memory and speed up the process.  Defaults to -99, for which there is no reduction in dimensionality..
+        :type repeat_dims: integer or list of 2 integers, optional
+        :param corr_axis: set to positive integer to select the axis used in the correlation matrix. The correlation matrix will then be averaged over other dimensions. Defaults to -99, for which the input array will be flattened and the full correlation matrix calculated.
+        :type corr_axis: integer, optional
+        :param fixed_corr_var: set to integer to copy the correlation matrix of the dimiension the integer refers to. Set to True to automatically detect if only one uncertainty is present and the correlation matrix of that dimension should be copied. Defaults to False.
+        :type fixed_corr_var: bool or integer, optional
+        :param output_vars: number of output parameters in the measurement function. Defaults to 1.
+        :type output_vars: integer, optional
+        :param PD_corr: set to True to make sure returned correlation matrices are positive semi-definite, default to True
+        :type PD_corr: bool, optional
+        :param refyvar: Index of output variable with reference shape (only relevant when output_vars>1; should be output variable with most dimensions; affects things like repeat_dims)
+        :type refyvar: int, optional
+        :return: uncertainties on measurand
+        :rtype: array
+        """
+        x = [np.atleast_1d(x[i]) for i in range(len(x))]
+        cov_x = [np.atleast_2d(cov_x[i]) for i in range(len(x))]
+        u_x = [
+            cm.uncertainty_from_covariance(cov_x[i]).reshape(x[i].shape)
+            for i in range(len(x))
+        ]
+        corr_x = [cm.correlation_from_covariance(cov_x[i]) for i in range(len(x))]
+
+        return self.propagate_standard(
+            func,
+            x,
+            u_x,
+            corr_x,
+            param_fixed=param_fixed,
+            corr_between=corr_between,
+            samples=samples,
+            return_corr=return_corr,
+            return_samples=return_samples,
+            repeat_dims=repeat_dims,
+            corr_axis=corr_axis,
+            fixed_corr_var=fixed_corr_var,
+            output_vars=output_vars,
+            PD_corr=PD_corr,
+            refyvar=refyvar,
+        )
+
     def propagate_standard(
         self,
         func,
@@ -233,6 +319,8 @@ class MCPropagation:
         :type param_fixed: list of bools, optional
         :param corr_between: correlation matrix (n,n) between input quantities, defaults to None
         :type corr_between: array, optional
+        :param samples: allows to provide a Monte Carlo sample previously generated. This sample of input quantities will be used instead of generating one from the uncertainties and error-correlation. Defaults to None
+        :type samples: list[array], optional
         :param return_corr: set to True to return correlation matrix of measurand, defaults to False
         :type return_corr: bool, optional
         :param return_samples: set to True to return generated samples, defaults to False
@@ -414,13 +502,14 @@ class MCPropagation:
                 output_vars,
             )
 
-    def propagate_cov(
+    def propagate_cov_flattened(
         self,
         func,
         x,
         cov_x,
         param_fixed=None,
         corr_between=None,
+        samples=None,
         return_corr=True,
         return_samples=False,
         repeat_dims=-99,
@@ -447,6 +536,8 @@ class MCPropagation:
         :type param_fixed: list of bools, optional
         :param corr_between: covariance matrix (n,n) between input quantities, defaults to None
         :type corr_between: array, optional
+        :param samples: allows to provide a Monte Carlo sample previously generated. This sample of input quantities will be used instead of generating one from the uncertainties and error-correlation. Defaults to None
+        :type samples: list[array], optional
         :param return_corr: set to True to return correlation matrix of measurand, defaults to True
         :type return_corr: bool, optional
         :param return_samples: set to True to return generated samples, defaults to False
@@ -584,48 +675,39 @@ class MCPropagation:
             )
 
             return outs
-            # return self.combine_repeated_outs(
-            #     outs,
-            #     yshapes,
-            #     len(x),
-            #     n_repeats,
-            #     repeat_shape,
-            #     repeat_dims,
-            #     return_corr,
-            #     return_samples,
-            #     output_vars,
-            #     refyvar,
-            # )
 
         else:
-            MC_data = np.empty(len(x), dtype=np.ndarray)
-            for i in range(len(x)):
-                if not hasattr(x[i], "__len__"):
-                    MC_data[i] = cm.generate_sample_systematic(
-                        self.MCsteps, x[i], cov_x[i]
-                    )
-                elif param_fixed is not None:
-                    if param_fixed[i] and (len(x[i].shape) == 2):
-                        MC_data[i] = np.array(
-                            [
-                                cm.generate_sample_cov(
-                                    self.MCsteps, x[i][:, j].ravel(), cov_x[i]
-                                ).reshape(x[i][:, j].shape + (self.MCsteps,))
-                                for j in range(x[i].shape[1])
-                            ]
-                        ).T
-                        MC_data[i] = np.moveaxis(MC_data[i], 0, 1)
+            if samples is not None:
+                MC_data = samples
+            else:
+                MC_data = np.empty(len(x), dtype=np.ndarray)
+                for i in range(len(x)):
+                    if not hasattr(x[i], "__len__"):
+                        MC_data[i] = cm.generate_sample_systematic(
+                            self.MCsteps, x[i], cov_x[i]
+                        )
+                    elif param_fixed is not None:
+                        if param_fixed[i] and (len(x[i].shape) == 2):
+                            MC_data[i] = np.array(
+                                [
+                                    cm.generate_sample_cov(
+                                        self.MCsteps, x[i][:, j].ravel(), cov_x[i]
+                                    ).reshape(x[i][:, j].shape + (self.MCsteps,))
+                                    for j in range(x[i].shape[1])
+                                ]
+                            ).T
+                            MC_data[i] = np.moveaxis(MC_data[i], 0, 1)
+                        else:
+                            MC_data[i] = cm.generate_sample_cov(
+                                self.MCsteps, x[i].ravel(), cov_x[i]
+                            ).reshape(x[i].shape + (self.MCsteps,))
                     else:
                         MC_data[i] = cm.generate_sample_cov(
                             self.MCsteps, x[i].ravel(), cov_x[i]
                         ).reshape(x[i].shape + (self.MCsteps,))
-                else:
-                    MC_data[i] = cm.generate_sample_cov(
-                        self.MCsteps, x[i].ravel(), cov_x[i]
-                    ).reshape(x[i].shape + (self.MCsteps,))
 
-            if corr_between is not None:
-                MC_data = cm.correlate_sample_corr(MC_data, corr_between)
+                if corr_between is not None:
+                    MC_data = cm.correlate_sample_corr(MC_data, corr_between)
 
         return self.process_samples(
             func,
@@ -786,20 +868,16 @@ class MCPropagation:
             repeat_shape = (n_repeats,)  # repeat_dims = -99
         else:
             repeat_dims = -np.sort(-np.array(repeat_dims))
-            n_repeats = yshape[repeat_dims[0]] * yshape[repeat_dims[1]]
-            repeat_shape = (yshape[repeat_dims[0]], yshape[repeat_dims[1]])
-            if corr_axis > repeat_dims[0]:
-                corr_axis -= 1
-            elif corr_axis == repeat_dims[0]:
-                raise ValueError(
-                    "punpy.mc_propagation: corr_axis and repeat_axis keywords should not be the same."
-                )
-            if corr_axis > repeat_dims[1]:
-                corr_axis -= 1
-            elif corr_axis == repeat_dims[1]:
-                raise ValueError(
-                    "punpy.mc_propagation: corr_axis and repeat_axis keywords should not be the same."
-                )
+            repeat_shape = tuple([yshape[repeat_dim] for repeat_dim in repeat_dims])
+            n_repeats = np.prod(repeat_shape)
+            for repeat_dim in repeat_dims:
+                if corr_axis > repeat_dim:
+                    corr_axis -= 1
+                elif corr_axis == repeat_dim:
+                    raise ValueError(
+                        "punpy.mc_propagation: corr_axis and repeat_axis keywords should not be the same."
+                    )
+
         return (
             yshapes,
             x,
@@ -1087,9 +1165,9 @@ class MCPropagation:
 
         else:
             if output_vars == 1:
-                u_func = u_func.reshape(repeat_shape + (-1,))
-                u_func = np.moveaxis(u_func, 0, repeat_dims[0])
-                u_func = np.moveaxis(u_func, 0, repeat_dims[1])
+                u_func = u_func.reshape(repeat_shape + u_func[0].shape)
+                for repeat_dim in repeat_dims:
+                    u_func = np.moveaxis(u_func, 0, repeat_dim)
             else:
                 u_funcb = u_func[:]
                 u_func = np.empty(output_vars, dtype=object)
